/*
  This command file will ensure that each user has a complete set of files in
  a specified directory tree.  For each *non-machine* user, this program will
  take a (supplied) source directory and use NSYNC to ensure that the user has
  the corresponding set of files in its subdirectory tree.

  "Non-machine," in this case, means a user that is listed from the NET USERS
  output and *NOT* listed in the NET GROUP RPLGROUP output (and is not this
  machine's requester name).

  Syntax:  UPDUSERS [source_tree] [target_tree] [([option] [option]...]

  where:   source_tree  overrides the default source tree
           target_tree  overrides the default target tree

           option is one of:
             DEBUG   --  to activate trace '?R' statements at major points
             NOEXEC  --  to prevent files from being copied
                           (overrides default behavior of copying files)
             TERSE   --  to suppress informational messages from the program
                           (overrides default behavior of displaying messages)

  Notes:  1) You must have the NSYNC(.EXE) utility available and in the PATH.
             If you do not have NSYNC, you can download it for free from the
             Device Driver Kit, at http://www.developer.ibm.com/ddk after
             accepting the license agreement (get the TOOLS.ZIP file).
          2) ALL PARAMETERS ARE OPTIONAL, and options can be in any order/case.
          3) If any options are specified, the collection of options must be
             preceded by a '(' character.

  Examples:
    UPDUSERS c:\common\files c:\ibmlan\dcdb\users (TERSE DEBUG
    UPDUSERS c:\users c:\ibmlan\dcdb\users

  All NSYNC output is redirected to <cmdfile>.LOG in the current directory,
  where <cmdfile> is the filename (e.g. UPDUSERS) of this command file.
*/
parse source . . me .                   /* Retrieve my fully-qualified name */
me=substr(me, lastpos('\', me)+1)       /* Get rid of path to this cmd file */
p=pos('.CMD', me)                       /* See if my filespec has a .CMD ext*/
if p>0 then                             /* If my filespec has the extension */
  me=delstr(me,p)                       /* Get rid of it as well            */

parse value '1 0 1' with verbose tracing doit .
parse value 'c:\srcfiles' 'c:\ibmlan\dcdb\users' with source_tree target_tree .

parse arg s_tree t_tree extra '('options
if s_tree='?' | extra<>'' then call help

if s_tree<>'' & s_tree<>'*' then
  source_tree=s_tree

if t_tree<>'' then
  target_tree=t_tree

do while options<>''
  parse upper var options opt options
  select
    when opt='DEBUG'  then tracing=1
    when opt='NOEXEC' then doit=0
    when opt='TERSE'  then verbose=0
    otherwise
      say "Invalid option '"opt"' specified"
      exit 100
  end
end

if tracing then trace '?R'

/*
  First, "drain" the console stack of any pre-existing lines.
*/
call drain_stack

'@ECHO OFF'

/*
  Retrieve the list of all users defined, by parsing the NET USERS output.
  Remember that there are three header lines and one trailer line that we are
  not interested in.
*/
if verbose then
  say 'Retrieving the list of users...'
users = get_list('NET USERS', 3, 1)

if tracing then trace '?R'

/*
  Now retrieve the list of machines from the NET GROUP RPLGROUP output.
*/
if verbose then
  say 'Retrieving the list of machines (RPLGROUP members)...'
machines = get_list('NET GROUP RPLGROUP', 5, 1)

/*
  Get the name of this server by parsing the NET CONFIG SERVER output, and add
  that name to the list of machines (it may already be in there, but this will
  not matter).
*/
if verbose then
  say 'Retrieving this machine''s name...'
'NET CONFIG SERVER | RXQUEUE'
parse pull . '\\'this_machine
machines=machines this_machine

/*
  We don't care about the rest of the lines from the 'NET CONFIG SERVER'
  output....
*/
call drain_stack

if tracing then trace '?R'

/*
  Now remove the machine names from the user list.
*/
if verbose then
  say 'Removing the machine names from the user names...'

list=machines
do while list<>''
  parse var list machine list
  i=wordpos(machine, users)
  if i>0 then do
    users=delword(users, i, 1)
  end
end

If verbose then
  say 'New user list is "'users'"'

/*
  The user names in the 'users' variable now represent true users and not
  (RIPL) machine names.
*/

if tracing then trace '?R'

/*
  Now, for each user, run NSYNC to ensure that all files from the source tree
  are also in the user's tree.  Only missing and newer files will be copied.

  Only issue the NSYNC command if execution is enabled (doit=1); otherwise,
  simply show the NSYNC command that would have been invoked.
*/
if verbose then
  say 'Updating the user trees...'

if doit=0 then noexec=' would have been invoked.'
          else noexec=''

user_list=users
do while user_list<>''
  parse var user_list user user_list    /* Get the next user from the list */
  if verbose then
    say '...Updating the user tree for "'user'"...'

  cmd='NSYNC' source_tree target_tree'\'user '-hrtw >>'me'.LOG 2>>&1'
  if doit=0 | verbose then
    say '......'cmd noexec

  if doit then
    ''cmd
end

if verbose then
  say 'User updates are complete!'
exit

/*
  The following procedure will execute the specified command, and return the
  list of all words contained in the output.  It will ignore the number of
  leading lines specified by "headers" and the number of trailing lines
  specified by "trailers."
*/
get_list:
  procedure
  parse arg cmd, headers, trailers
  cmd' | RXQUEUE'
  q=queued()
  do i=1-headers to q-headers
    parse pull line.i
  end
  output = ''
  do i=1 to q-headers-trailers
    output = output line.i
  end
  output = space(output, 1)
  return output

/*
  The following procedure simply clears ("drains") the program stack, by first
  querying how many lines are in the stack and then pulling each one.
*/
drain_stack:
  procedure
  q=queued()
  do i=1 to q; parse pull .;end
  return

help:
  procedure expose me source_tree target_tree
  say "Syntax:  "me" [source_tree] [target_tree] [([option] [option]...]"
  say
  say "where:   source_tree  overrides default source tree of """source_tree'"'
  say "         target_tree  overrides default target tree of """target_tree'"'
  say
  say "         option is one of:"
  say "           DEBUG   --  to activate trace '?R' statements at major points"
  say "           NOEXEC  --  to prevent files from being copied"
  say "                         (overrides default behavior of copying files)"
  say "           TERSE   --  to suppress informational messages from the program"
  say "                         (overrides default behavior of displaying messages)"
  say
  say "Notes:  1) You must have the NSYNC(.EXE) utility available and in the PATH."
  say "           If you do not have NSYNC, you can download it for free from the"
  say "           Device Driver Kit, at http://www.developer.ibm.com/ddk after"
  say "           accepting the license agreement (get the TOOLS.ZIP file)."
  say "        2) ALL PARAMETERS ARE OPTIONAL, and options can be in any order/case."
  say "        3) If any options are specified, the collection of options must be"
  say "           preceded by a '(' character."
  say
  say "Examples:"
  say "  "me" c:\common\files c:\ibmlan\dcdb\users (TERSE DEBUG"
  say "  "me" c:\users c:\ibmlan\dcdb\users"
  exit 100
